/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { CellError, InternalNoErrorCellValue, InternalScalarValue } from '../Cell';
import { Config } from '../Config';
import { DateTimeHelper } from '../DateTimeHelper';
import { Maybe } from '../Maybe';
import { NumberLiteralHelper } from '../NumberLiteralHelper';
import { InterpreterValue, SimpleRangeValue } from './InterpreterValue';
export declare class ArithmeticHelper {
    private readonly config;
    private readonly dateTimeHelper;
    private readonly numberLiteralsHelper;
    private readonly collator;
    private readonly actualEps;
    constructor(config: Config, dateTimeHelper: DateTimeHelper, numberLiteralsHelper: NumberLiteralHelper);
    eqMatcherFunction(pattern: string): (arg: InterpreterValue) => boolean;
    neqMatcherFunction(pattern: string): (arg: InterpreterValue) => boolean;
    searchString(pattern: string, text: string): number;
    requiresRegex(pattern: string): boolean;
    private buildRegex;
    private normalizeString;
    compare(left: InternalNoErrorCellValue, right: InternalNoErrorCellValue): number;
    floatCmp(left: number, right: number): number;
    stringCmp(left: string, right: string): number;
    addWithEpsilon: (left: number, right: number) => number;
    /**
     * Adds two numbers
     *
     * Implementation of adding which is used in interpreter.
     *
     * @param left - left operand of addition
     * @param right - right operand of addition
     */
    nonstrictadd: (left: InternalScalarValue, right: InternalScalarValue) => number | CellError;
    /**
     * Subtracts two numbers
     *
     * Implementation of subtracting which is used in interpreter.
     *
     * @param left - left operand of subtraction
     * @param right - right operand of subtraction
     * @param eps - precision of comparison
     */
    subtract: (left: number, right: number) => number;
    coerceScalarToNumberOrError(arg: InternalScalarValue): number | CellError;
    coerceToMaybeNumber(arg: InternalScalarValue): Maybe<number>;
    coerceNonDateScalarToMaybeNumber(arg: InternalScalarValue): Maybe<number>;
}
export declare function coerceToRange(arg: InterpreterValue): SimpleRangeValue;
export declare function coerceToRangeNumbersOrError(arg: InterpreterValue): SimpleRangeValue | CellError | null;
export declare function coerceBooleanToNumber(arg: boolean): number;
export declare function coerceEmptyToValue(arg: InternalNoErrorCellValue): InternalNoErrorCellValue;
/**
 * Coerce scalar value to boolean if possible, or error if value is an error
 *
 * @param arg
 */
export declare function coerceScalarToBoolean(arg: InternalScalarValue): boolean | CellError | undefined;
export declare function coerceScalarToString(arg: InternalScalarValue): string | CellError;
export declare function divide(left: number, right: number): number | CellError;
export declare function numberCmp(left: number, right: number): number;
export declare function isNumberOverflow(arg: number): boolean;
export declare function fixNegativeZero(arg: number): number;
